package net.dries007.holoInventory.client;

import com.google.common.cache.Cache;
import com.google.common.cache.CacheBuilder;
import net.dries007.holoInventory.Helper;
import net.dries007.holoInventory.HoloInventory;
import net.dries007.holoInventory.client.renderers.IRenderer;
import net.dries007.holoInventory.network.request.EntityRequest;
import net.dries007.holoInventory.network.request.TileRequest;
import net.minecraft.client.Minecraft;
import net.minecraft.client.multiplayer.WorldClient;
import net.minecraft.client.settings.GameSettings;
import net.minecraft.client.settings.KeyBinding;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.util.math.Vec3d;
import net.minecraftforge.client.event.RenderWorldLastEvent;
import net.minecraftforge.client.settings.KeyConflictContext;
import net.minecraftforge.client.settings.KeyModifier;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.fml.client.registry.ClientRegistry;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.common.gameevent.InputEvent;
import net.minecraftforge.fml.common.gameevent.TickEvent;
import org.lwjgl.input.Keyboard;

import java.lang.ref.WeakReference;
import java.text.DecimalFormat;
import java.util.concurrent.TimeUnit;

public class ClientEventHandler
{
    public static final DecimalFormat DF = new DecimalFormat("#.#k");
    public static final int TEXTCOLOR = 255 + (255 << 8) + (255 << 16) + (170 << 24);

    private static final Cache<BlockPos, IRenderer> TILE_CACHE = CacheBuilder.newBuilder().maximumSize(150).expireAfterWrite(500, TimeUnit.MILLISECONDS).build();
    private static final Cache<Integer, IRenderer> ENTITY_CACHE = CacheBuilder.newBuilder().maximumSize(150).expireAfterWrite(500, TimeUnit.MILLISECONDS).build();

    private static ClientEventHandler instance;
    private static boolean enabled = true;
    private static boolean toggleState = true;
    private static KeyBinding keyHold;
    private static KeyBinding keyToggle;

    private WeakReference<WorldClient> worldRef = new WeakReference<>(null);

    private ClientEventHandler()
    {

    }

    public static void init()
    {
        if (instance != null) MinecraftForge.EVENT_BUS.unregister(instance);
        instance = new ClientEventHandler();
        MinecraftForge.EVENT_BUS.register(new ClientEventHandler());

        keyHold = new KeyBinding("key.ho.hold", KeyConflictContext.IN_GAME, Keyboard.KEY_H, HoloInventory.MODID);
        keyToggle = new KeyBinding("key.ho.toggle", KeyConflictContext.IN_GAME, 0, HoloInventory.MODID);
        ClientRegistry.registerKeyBinding(keyHold);
        ClientRegistry.registerKeyBinding(keyToggle);
    }

    public static void cache(BlockPos pos, IRenderer iRenderer)
    {
        TILE_CACHE.put(pos, iRenderer);
    }

    public static void cache(int id, IRenderer iRenderer)
    {
        ENTITY_CACHE.put(id, iRenderer);
    }

    @SubscribeEvent
    public void onKeyInput(InputEvent.KeyInputEvent event)
    {
        if (keyToggle.func_151468_f()) toggleState = !toggleState;
        if (keyHold.func_151470_d()) enabled = true;
    }

    @SubscribeEvent
    public void updateEvent(TickEvent.ClientTickEvent event)
    {
        if (event.phase == TickEvent.Phase.END) return;
        Minecraft mc = Minecraft.func_71410_x();
        if (mc == null || mc.func_147113_T()) return;
        WorldClient mcWorld = Minecraft.func_71410_x().field_71441_e;
        if (mcWorld == null) return;

        WorldClient world = worldRef.get();
        if (world == null || mcWorld != world)
        {
            TILE_CACHE.invalidateAll();
            ENTITY_CACHE.invalidateAll();
            worldRef = new WeakReference<>(mcWorld);
            return;
        }

        enabled = (Helper.showOnSneak && mc.field_71474_y.field_74311_E.func_151470_d()) ||
                (Helper.showOnSprint && mc.field_71474_y.field_151444_V.func_151470_d()) ||
                (keyHold.func_151463_i() != 0 && keyHold.func_151470_d()) ||
                (keyToggle.func_151463_i() != 0 && toggleState);

        if (!enabled) return;

        TILE_CACHE.cleanUp();
        ENTITY_CACHE.cleanUp();

        RayTraceResult ray = mc.field_71476_x;
        if (ray == null) return;

        if (ray.field_72313_a == RayTraceResult.Type.BLOCK)
        {
            final TileEntity tileEntity = world.func_175625_s(ray.func_178782_a());
            if (tileEntity == null || !Helper.accept(tileEntity)) return;
            HoloInventory.getSnw().sendToServer(new TileRequest(world.field_73011_w.getDimension(), ray.func_178782_a()));
        }
        else if (ray.field_72313_a == RayTraceResult.Type.ENTITY && Helper.accept(ray.field_72308_g))
        {
            HoloInventory.getSnw().sendToServer(new EntityRequest(world.field_73011_w.getDimension(), ray.field_72308_g.func_145782_y()));
        }
    }

    @SubscribeEvent
    public void renderEvent(RenderWorldLastEvent event)
    {
        if (worldRef.get() == null || !enabled) return;
        Minecraft mc = Minecraft.func_71410_x();
        RayTraceResult ray = mc.field_71476_x;
        if (ray == null) return;

        if (ray.field_72313_a == RayTraceResult.Type.BLOCK)
        {
            final IRenderer renderer = TILE_CACHE.getIfPresent(ray.func_178782_a());
            if (renderer == null) return;
            try
            {
                renderer.render(worldRef.get(), ray, ray.field_72307_f);
            }
            catch (Exception e)
            {
                HoloInventory.getLogger().warn("Some error while rendering the hologram :(");
                HoloInventory.getLogger().warn("INFO: Block @ {}", ray.func_178782_a());
                HoloInventory.getLogger().warn("Please make an issue on github if this happens.");
                HoloInventory.getLogger().catching(e);
            }
        }
        else if (ray.field_72313_a == RayTraceResult.Type.ENTITY)
        {
            final IRenderer renderer = ENTITY_CACHE.getIfPresent(ray.field_72308_g.func_145782_y());
            if (renderer == null) return;
            try
            {
                renderer.render(worldRef.get(), ray, ray.field_72307_f);
            }
            catch (Exception e)
            {
                HoloInventory.getLogger().warn("Some error while rendering the hologram :(");
                HoloInventory.getLogger().warn("INFO: Entity: {}", ray.field_72308_g);
                HoloInventory.getLogger().warn("Please make an issue on github if this happens.");
                HoloInventory.getLogger().catching(e);
            }
        }
    }
}
